//
//  XMLRPCDecoder.m
//  XMLRPC
//
//  Created by Russell Jancewicz on 2012-05-29.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "XMLRPCDecoder.h"

static NSString * const kXMLRPCMethodResponse = @"methodResponse";
static NSString * const kXMLRPCParams =         @"params";
static NSString * const kXMLRPCParam =          @"param";
static NSString * const kXMLRPCFault =          @"fault";
static NSString * const kXMLRPCValue =          @"value";

static NSString * const kXMLRPCBoolean =        @"boolean";
static NSString * const kXMLRPCInt =            @"int";
static NSString * const kXMLRPCI4 =             @"i4";
static NSString * const kXMLRPCDouble =         @"double";
static NSString * const kXMLRPCString =         @"string";
static NSString * const kXMLRPCISO8601 =        @"dateTime.iso8601";
static NSString * const kXMLRPCBase64 =         @"base64";

static NSString * const kXMLRPCArray =          @"array";
static NSString * const kXMLRPCData =           @"data";

static NSString * const kXMLRPCStruct =         @"struct";
static NSString * const kXMLRPCMember =         @"member";
static NSString * const kXMLRPCName =           @"name";


@interface XMLRPCDecoder ()

@property (strong, nonatomic) NSString *stringValue;

@end

@implementation XMLRPCDecoder 

@synthesize name = _name;
@synthesize value = _value;
@synthesize parent = _parent;
@synthesize parser = _parser;
@synthesize fault = _fault;

@synthesize stringValue = _stringValue;



- (id)initWithData:(NSData *)data {
    self = [super init];
    if (self) {
        self.parser = [[NSXMLParser allocWithZone:NULL] initWithData:data];
        self.parser.delegate = self;

        self.fault = NO;
        self.parent = nil;
        self.value = nil;
        self.name = nil;
    }
    return self;
}

- (id)decode {
    [self.parser parse];
    
    return self.value;
}

- (id)initWithParent:(XMLRPCDecoder *)parent {
    
    self = [super init];
    if (self) {
        self.parser = parent.parser;
        self.parser.delegate = self;

        self.fault = NO;
        self.parent = parent;
        self.value = nil;
        self.name = nil;
    }
    return self;
}

+ (id)parserWithParent:(XMLRPCDecoder *)parent {
    return [[XMLRPCDecoder allocWithZone:NULL] initWithParent:parent];
}


/* XML Parser */

- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)string {
    
    self.stringValue = string;
    
}

- (void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName attributes:(NSDictionary *)attributeDict {
    
    /*
        Changed the order here (value is most likely node to exist so it makes since to check for it first)
     */
    if ([elementName isEqualToString:kXMLRPCValue] || [elementName isEqualToString:kXMLRPCMember]) {
        [XMLRPCDecoder parserWithParent:self];        
    } else if ([elementName isEqualToString:kXMLRPCFault]) {
        _fault = YES;
    } else if ([elementName isEqualToString:kXMLRPCArray]) {
        self.value = [NSMutableArray array];
    } else if ([elementName isEqualToString:kXMLRPCStruct]) {
        self.value = [NSMutableDictionary dictionary];
    }  
    
}

- (id)parseType:(NSString *)type fromString:(NSString *)string {
    
    //- (id)parseString:(NSString *)string intoType:(NSString *)type {
    
    if ([type isEqualToString:kXMLRPCInt] || [type isEqualToString:kXMLRPCI4]) {
        return [NSNumber numberWithInteger:[string integerValue]];
    } else if ([type isEqualToString:kXMLRPCBoolean]) {
        return [NSNumber numberWithBool:[string boolValue]];
    } else if ([type isEqualToString:kXMLRPCString]) {
        return string;
    } else if ([type isEqualToString:kXMLRPCDouble]) {
        return [NSNumber numberWithDouble:[string doubleValue]];
    } else if ([type isEqualToString:kXMLRPCISO8601]) {
        
        NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
        [formatter setDateFormat:@"yyyyMMdd'T'HH:mm:ss"];
        
        return [formatter dateFromString:string];
    } else if ([type isEqualToString:kXMLRPCBase64]) {
        // todo
    }
    
    NSLog(@"%@", type);
    //    NSLog()
    
    return [NSNull null];
}

- (void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName {
    
    //NSLog(@"elementName, %@", elementName);
    
    if ([elementName isEqualToString:kXMLRPCValue] || [elementName isEqualToString:kXMLRPCMember]) {
        
        if (self.parent) {
            self.parser.delegate = self.parent; 
            
            if ([self.parent.value isKindOfClass:[NSMutableArray class]]) {
                [(NSMutableArray *)self.parent.value addObject:self.value];
            } else if ([self.parent.value isKindOfClass:[NSMutableDictionary class]]) {
                if (self.name) {                    
                    if (self.value == nil) self.value = [NSNull null];
                    [(NSMutableDictionary *)self.parent.value setObject:self.value forKey:self.name];
                }
            } else {
                self.parent.value = self.value;
            }
        }
        
    } else if ([elementName isEqualToString:kXMLRPCArray] || [elementName isEqualToString:kXMLRPCStruct]) {
    } else if ([elementName isEqualToString:kXMLRPCName]) {
        
        self.name = self.stringValue;
        
    } else {
        
        if (self.value == nil) {
            if (self.stringValue == nil) self.stringValue = @"";
            self.value = [self parseType:elementName fromString:self.stringValue];
        }
        
    }
    
}

- (void)parserDidEndDocument:(NSXMLParser *)parser {
    
}


@end
